//a class for tracking hits on an actor
class HitList
{
	private array<actor> HitA;
	
	//can't have an array of vector3 so have to do it like this
	private array<float> HitX;
	private array<float> HitY;
	private array<float> HitZ;

	//tells how many hits on the list
	uint Size() {return HitZ.Size();}

	//clears everything
	void clear()
	{
		HitA.clear();
		HitX.clear();
		HitY.clear();
		HitZ.clear();
	}

	//adds a hit to the list
	uint Push3Floats(actor HitActor, float HitLocationX, float HitLocationY, float HitLocationZ)
	{
		if (!HitActor) return -1;
		
		HitA.Push(HitActor);
		HitX.Push(HitLocationX);
		HitY.Push(HitLocationY);
		HitZ.Push(HitLocationZ);
		return HitX.Size() - 1;
	}
	
	//adds a hit to the list
	uint PushVector3(actor HitActor, vector3 HitLocation)
	{
		return Push3Floats(HitActor, HitLocation.x, HitLocation.y, HitLocation.z);
	}
	
	//gets a specific hit
	actor, vector3 Get(uint IndexNumber)
	{
		return HitA[IndexNumber], (HitX[IndexNumber], HitY[IndexNumber], HitZ[IndexNumber]);
	}
	
	//sorts the hits by their distance to a point
	void SortByDistanceTo(vector3 ReferencePos)
	{
		if (Size() <= 1) {return;}
	
		array<double> ValueList;
		array<int> IndexList;
		
		for (int i = 0; i < size(); i++)
		{
			vector3 diff = (HitX[i], HitY[i], HitZ[i]) - ReferencePos;
			//TODO: is "dot" faster than "length()"? Need to test this
			//ALT: ValueList.push( diff.length() );
			ValueList.push( diff dot diff );
			IndexList.push(i);
		}
		
		ArraySort.SortDouble(ValueList, IndexList);
		
		array<actor> newHitA;
		array<float> newHitX;
		array<float> newHitY;
		array<float> newHitZ;
		
		for (int i = 0; i < IndexList.size(); i++)
		{
			newHitA.push(HitA[IndexList[i]]);
			newHitX.push(HitX[IndexList[i]]);
			newHitY.push(HitY[IndexList[i]]);
			newHitZ.push(HitZ[IndexList[i]]);
		}
		
		HitA.move(newHitA);
		HitX.move(newHitX);
		HitY.move(newHitY);
		HitZ.move(newHitZ);
	}

	void print()
	{
		for (int i = 0; i < HitZ.size(); i++)
		{
			console.printf("HitList: (%8.2f,%8.2f,%8.2f) %s", HitX[i], HitY[i], HitZ[i], HitA[i].GetCharacterName());
		}
	}
}



struct LineSegCollide play
{
	static
	bool CheckLineSegHitsActor(vector3 LineStart, vector3 LineStop, actor CheckTarget, out HitList Hits)
	{
		double aLeft = CheckTarget.pos.x - CheckTarget.radius;
		double aRight = CheckTarget.pos.x + CheckTarget.radius;
		double aFront = CheckTarget.pos.y - CheckTarget.radius;
		double aBack = CheckTarget.pos.y + CheckTarget.radius;
		double aBottom = CheckTarget.pos.z;
		double aTop = CheckTarget.pos.z + CheckTarget.height;

		//check for easy misses
		if (LineStart.x < aLeft && LineStop.x < aLeft) return false;
		if (LineStart.x > aRight && LineStop.x > aRight) return false;
		if (LineStart.y < aFront && LineStop.y < aFront) return false;
		if (LineStart.y > aBack && LineStop.y > aBack) return false;
		if (LineStart.z < aBottom && LineStop.z < aBottom) return false;
		if (LineStart.z > aTop && LineStop.z > aTop) return false;
		
		vector3 LineDirection = LineStop - LineStart;

		if (!Hits) Hits = new("HitList");
		int NumHits = 0;
	
		//check left
		if (LineDirection.x != 0)
		{
			double t = (aLeft - LineStart.x) / LineDirection.x;
			if (0 <= t && t <= 1)
			{
				double y = LineStart.y + LineDirection.y * t;
				double z = LineStart.z + LineDirection.z * t;
				if (aFront <= y && y <= aBack && aBottom <= z && z <= aTop)
				{
					Hits.Push3Floats(CheckTarget, aLeft, y, z);
					NumHits++;
				}
			}
		}

		//check right
		if (LineDirection.x != 0)
		{
			double t = (aRight - LineStart.x) / LineDirection.x;
			if (0 <= t && t <= 1)
			{
				double y = LineStart.y + LineDirection.y * t;
				double z = LineStart.z + LineDirection.z * t;
				if (aFront <= y && y <= aBack && aBottom <= z && z <= aTop)
				{
					Hits.Push3Floats(CheckTarget, aRight, y, z);
					NumHits++;
				}
			}
		}
		
		//check front
		if (LineDirection.y != 0)
		{
			double t = (aFront - LineStart.y) / LineDirection.y;
			if (0 <= t && t <= 1)
			{
				double x = LineStart.x + LineDirection.x * t;
				double z = LineStart.z + LineDirection.z * t;
				if (aLeft <= x && x <= aRight && aBottom <= z && z <= aTop)
				{
					Hits.Push3Floats(CheckTarget, x, aFront, z);
					NumHits++;
				}
			}
		}

		//check back
		if (LineDirection.y != 0)
		{
			double t = (aBack - LineStart.y) / LineDirection.y;
			if (0 <= t && t <= 1)
			{
				double x = LineStart.x + LineDirection.x * t;
				double z = LineStart.z + LineDirection.z * t;
				if (aLeft <= x && x <= aRight && aBottom <= z && z <= aTop)
				{
					Hits.Push3Floats(CheckTarget, x, aBack, z);
					NumHits++;
				}
			}
		}

		//check bottom
		if (LineDirection.z != 0)
		{
			double t = (aBottom - LineStart.z) / LineDirection.z;
			if (0 <= t && t <= 1)
			{
				double x = LineStart.x + LineDirection.x * t;
				double y = LineStart.y + LineDirection.y * t;
				if (aLeft <= x && x <= aRight && aFront <= y && y <= aBack)
				{
					Hits.Push3Floats(CheckTarget, x, y, aBottom);
					NumHits++;
				}
			}
		}

		//check top
		if (LineDirection.z != 0)
		{
			double t = (aTop - LineStart.z) / LineDirection.z;
			if (0 <= t && t <= 1)
			{
				double x = LineStart.x + LineDirection.x * t;
				double y = LineStart.y + LineDirection.y * t;
				if (aLeft <= x && x <= aRight && aFront <= y && y <= aBack)
				{
					Hits.Push3Floats(CheckTarget, x, y, aTop);
					NumHits++;
				}
			}
		}
		
		return NumHits > 0;
	}
	
	static
	bool CheckLineSegHitsActors(vector3 LineStart, vector3 LineStop, array<actor> CheckTargetList, out HitList Hits)
	{
		if (!Hits) Hits = new("HitList");
	
		bool ret = false;
	
		for (int i = 0; i < CheckTargetList.size(); i++)
		{
			if (!CheckTargetList[i]) continue;
			
			if (CheckLineSegHitsActor(LineStart, LineStop, CheckTargetList[i], Hits)) {ret = true;}
		}
		
		return ret;
	}
	
	static
	bool CheckPawnLosHitList(PlayerPawn pp, array<actor> CheckTarget, out HitList Hits, double LookRange = 4096, int TRF_Flags = 0, out FLineTraceData ltd = null)
	{
		if (!pp) return false;
		
		pp.LineTrace(pp.Angle, LookRange, pp.Pitch, TRF_flags, pp.player.viewheight, data: ltd);
		vector3 LineStart = pp.pos;
		LineStart.z += pp.player.viewheight;
		vector3 LineStop = ltd.HitLocation;
		
		bool ret = CheckLineSegHitsActors(LineStart, LineStop, CheckTarget, Hits);
		Hits.SortByDistanceTo(LineStart);

		return ret;
	}

	static
	bool CheckPlayerLineOfSightHitList(int PlayNum, array<actor> CheckTarget, out HitList Hits, double LookRange = 4096, int TRF_Flags = 0, out FLineTraceData ltd = null)
	{
		if (PlayNum < 0 || PlayNum - 1 > Players.size() || !PlayerInGame[PlayNum]) return false;

		PlayerPawn pp = players[PlayNum].mo;
		if (!pp) return false;
		
		return CheckPawnLosHitList(pp, CheckTarget, Hits, LookRange, TRF_Flags);
	}
	
	static
	actor, vector3 CheckPawnLosHitActor(PlayerPawn pp, array<actor> CheckTarget, double LookRange = 4096, int TRF_flags = 0)
	{
		if (!pp) return null, (0,0,0);
		
		actor retAct = null;
		vector3 retPos = pp.pos + (0,0,pp.player.ViewHeight);
		fLineTraceData ltd;
		HitList Hits = new("HitList");
		if (CheckPawnLosHitList(pp, CheckTarget, Hits, LookRange, TRF_Flags, ltd))
		{
			Hits.SortByDistanceTo(retPos);
		}
		if (0 < Hits.size())
		{
			[retAct, retPos] = Hits.Get(0);
		}
		else if (ltd.HitType == TRACE_HitActor)
		{
			retAct = ltd.HitActor;
			retPos = ltd.HitLocation;
		}
		else
		{
			retPos = ltd.HitLocation;
		}
		
		return retAct, retPos;
	}
	
	static
	actor, vector3 CheckPlayerLineOfSightHitActor(int PlayNum, array<actor> CheckTarget, double LookRange = 4096, int TRF_flags = 0)
	{
		if (PlayNum < 0 || PlayNum - 1 > Players.size() || !PlayerInGame[PlayNum]) return null, (0,0,0);

		PlayerPawn pp = players[PlayNum].mo;
		if (!pp) return null, (0,0,0);
		
		return CheckPawnLosHitActor(pp, CheckTarget, LookRange, TRF_Flags);
	}
}
