class LootCursorInv : MWR_Cursor3dInv
{
	default
	{
		MWR_Cursor3dInv.Cursor2DClassName 'MarkerCursor';
	}
	
	PositionMarker _cursorPos;
	array<actor> PosMarkList; //should be PositionMarker type but I need this to be generic for later
	
	actor GetCursorTarget()
	{
		actor ret = Super.GetCursorTarget();
		return ret;
	}
	
	override void DoWhileEnabled()
	{
		actor TargetActor;
		vector3 TargetPos;
		[TargetActor, TargetPos] = LineSegCollide.CheckPawnLosHitActor(PlayerPawn(owner), PosMarkList, TRF_Flags: TraceFlags);
		
		MarkerCursor mcursor = MarkerCursor(_cursor2d);
		
		if (TargetActor)
		{
			//update the 2d cursor, it make have hit a nonblockmap target
			if (mcursor)
			{
				mcursor.SetOrigin(TargetPos, true);
				mcursor.bWallSprite = false;
				mcursor.bFlatSprite = false;
			}

			//Spawn 3d cursor if it doesn't already exist
			if (!_cursor3d) _cursor3d = MWR_cursor3d(Spawn(Cursor3DClassName, TargetActor.pos));

			//update the 3d cursor, it make have hit a nonblockmap target
			if (_cursor3d)
			{
				//set position
				if (_cursor3d.pos != TargetActor.pos) _cursor3d.SetOrigin(TargetActor.pos, true);

				//set size
				double r = TargetActor.Radius;
				double h = TargetActor.Height;
				if (r!=_cursor3d.radius || h!=_cursor3d.height) _cursor3d.A_SetSize(r, h, false);
			}

			//update the cursors' colors based on target
			if (LootMarker.FindAttached(TargetActor))
			{
				if (_cursor3d) _cursor3d.SetColorRed();
				if (mcursor) mcursor.SetColorRed();
			} else {
				if (_cursor3d) _cursor3d.SetColorGreen();
				if (mcursor) mcursor.SetColorGreen();
			}
			
			//See the 2d cursor
			if (mcursor) mcursor.bInvisible = false;
		}
		else
		{
			//See the 2d cursor if not using position markers
			if (mcursor)
			{
				mcursor.bFlatSprite = true;
				mcursor.SetColorBlue();
				mcursor.bInvisible = LootMarkersUsePos;
			}
		}
		
		//handle position cursor
		if (LootMarkersUsePos && !TargetActor && bCursorHit && (tCursorHit.HitType == TRACE_HitWall || tCursorHit.HitType == TRACE_HitFloor || tCursorHit.HitType == TRACE_HitCeiling))
		{
			if (!_cursorPos) _cursorPos = PositionMarker(spawn("PositionMarker", _cursor2d.pos));
			if (tCursorHit.HitType == TRACE_HitWall)
			{
				_cursorPos.bWallSprite = true;
				_cursorPos.angle = _cursor2d.angle;
			}
			else
			{
				_cursorPos.bWallSprite = false;
			}
			_cursorPos.MarkSpot(_cursor2d.pos, LootMarkersNumber, LootMarkersColor);
		}
		else
		{
			if (_cursorPos) _cursorPos.Destroy();
		}
	}

	override void DoBeforeDisable()
	{
		//add or remove marker

		actor TargetActor;
		vector3 TargetPos;
		[TargetActor, TargetPos] = LineSegCollide.CheckPawnLosHitActor(PlayerPawn(owner), PosMarkList, TRF_Flags: TraceFlags);

		if (TargetActor)
		{
			string msg;
			if (TargetActor is "PositionMarker")
			{
				//remove it from the list
				int i = PosMarkList.find(TargetActor);
				if (i < PosMarkList.size())
				{
					PosMarkList.Delete(i);
				}
				
				//remove the map marker
				LootMarker.RemoveAttachedFrom(TargetActor);

				//remove the position marker
				TargetActor.Destroy();

				//announce the lost position marker
				msg=StringTable.Localize("$REMMARKMAP");
			}
			else
			{
				if (LootMarker.FindAttached(TargetActor))
				{
					//remove the marker from the target and prepare announcement
					LootMarker.RemoveAttachedFrom(TargetActor);
					msg=StringTable.Localize("$REMMARK");
					msg.replace('%a',TargetActor.GetCharacterName());
				}
				else
				{
					//add a marker to the target and prepare announcement
					LootMarker.AttachOneTo(TargetActor);
					msg=StringTable.Localize("$ADDMARK");
					msg.replace('%a',TargetActor.GetCharacterName());
				}
			}
			//make the announcement
			console.printf(msg);
		}
		
		//handle position cursor
		if (_cursorPos)
		{
			LootMarker marker;
			marker = LootMarker.AttachOneTo(_cursorPos);
			marker.args[2]=0;
			marker = LootMarker.AttachOneTo(_cursorPos.myNumber);
			marker.args[2]=0;
			console.printf(StringTable.Localize("$ADDMARKMAP"));
			PosMarkList.push(_cursorPos);
			_cursorPos=null;
			BumpMarkerNumber();
			
			//console.printf("TryIt?");//DEBUG
			//LineHitsBox.TryIt(owner);
		}
	}
	
	static void BumpMarkerNumber()
	{
		Cvar LootMarkersNumber = CVar.FindCvar("LootMarkersNumber");
		if (LootMarkersNumber)
		{
			int newNumber = (LootMarkersNumber.GetInt() + 1) % 100;
			LootMarkersNumber.SetInt(newNumber);
		}
	}
}

class LootCursorHandler : EventHandler{
	override void NetworkProcess(ConsoleEvent e){
		if (e.Name ~== "mwr_lootcursor_on")
		{
			SetCursorEnabled(e.Player, true);
		}
		else if (e.Name ~== "mwr_lootcursor_off")
		{
			SetCursorEnabled(e.Player, false);
		}
		else if (e.Name ~== "mwr_lootcursor_color")
		{
			BumpMarkerColor();
		}
	}

	private void SetCursorEnabled(int player, bool enabled){
		if (playeringame[player]){
			let myInv = LootCursorInv(players[player].mo.FindInventory('LootCursorInv'));
			
			if (myInv == null){
				players[player].mo.GiveInventory('LootCursorInv',1);
				myInv = LootCursorInv(players[player].mo.FindInventory('LootCursorInv'));
			}
			

			if (myInv != null){
				if (enabled){
					myInv.EnableCursor();
				} else {
					myInv.DisableCursor();
				}
			} else {
				console.printf('[ERROR]No loot cursor in inventory');
			}
		}
	}
	
	private void BumpMarkerColor()
	{
		Cvar LootMarkersColor = CVar.FindCvar("LootMarkersColor");
		if (LootMarkersColor)
		{
			int c = (LootMarkersColor.GetInt() + 1) % 8;
			//console.printf("Loot Marker: Cycled color to %d", c);//DBUG
			LootMarkersColor.SetInt(c);
			
			array<string> ColorNames;
			StringTable.Localize("$MARKCOLORLIST").split(ColorNames, ",", TOK_KEEPEMPTY);
			string colorName="[UNKNOWN]";
			if (0 <= c && c <= ColorNames.size()) colorName = colorNames[c];
			string message=StringTable.Localize("$MARKCOLORSET");
			message.replace("%c",colorName);
			console.printf(message);
		}
	}
}

class MarkerCursor : MWR_Cursor2d
{
	default
	{
		//why are these not inherited from MWR_Cursor2d?
		+NOINTERACTION
		+NOBLOCKMAP
	}
	states
	{
		spawn:
		blue:
			LMCS B -1 bright;
			stop;
		green:
			LMCS G -1 bright;
			stop;
		red:
			LMCS R -1 bright;
			stop;
	}
	
	void SetColorBlue()
	{
		SetState(ResolveState("blue"));
	}

	void SetColorGreen()
	{
		SetState(ResolveState("green"));
	}

	void SetColorRed()
	{
		SetState(ResolveState("red"));
	}
}

